/*
RC Massey 8680 Tractor Cab Code Version 2.0
 This code was written by Oisin O'Conchubhair for the website www.rctractors.net
 This is the generic code used to control homemade RC Tractors.
 The code is intended to be used with an Arduino Pro Mini
 Pin Assignment
 D0 Serial RX
 D1 Serial TX
 D2 Right Indicator
 D3 Front Work Lights
 D4 Right Beacon
 D5 Rear Work Lights
 D6 
 D7 Left Beacon
 D8 NRF24 CE
 D9 Left Indicator
 D10 NRF24 CSN
 D11 NRF24 MOSI
 D12 NRF24 MISO
 D13 NRF24 SCK
 A0 Head Lights 
 A1 
 A2 Not assigned
 A3 Not assigned
 A4 Not assigned
 A5 Not assigned
 */

// This section describes the vehicle

// Variables used in the program
byte ID = 3; // Enter a four byte ID for the vehicle
int Active = 0; // Variable to determine if the tractor is in active
int Last_Data_Timer = 0;
int Controller_Locked = 0;
int Receive_Count = 0;

int Controller_ID = 0;

uint8_t buf[16]; // Array to store recieved data

// Include Libraries
#include <SPI.h>
#include "RCTractorGuyRF24.h"
#include "RCTractorGuy.h"

// Initialise NRF24L01
RF24 radio(8,10);

// Initialise Tractor
/*(Motor_1_PWM, Motor_1_Dir_1, Motor_1_Dir_2, Motor_2_PWM, Motor_2_Dir_1, Motor_2_Dir_2,
Steering, Rear_Link, Front_Link, Left_Ind, Right_Ind, Head_Lights, Tail_Lights, Work_Lights,
Accessory_Lights, Left_Beacon, Right_Beacon)*/
RCTractor JD8360RT(0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 2, 0, 0, 3, 0, 0, 0);

// Controller Address
const uint64_t Controller[5] = { 0xE8E8F0F0E1LL, 0xE8E8F0F0E2LL, 0xE8E8F0F0E3LL, 0xE8E8F0F0E4LL, 0xE8E8F0F0E5LL};

void setup() 
{
	  // Start the NRF24L01
	  radio.begin();

	  // Enable Ack Payloads to automatically send data back with the Acknowledge
	  radio.enableAckPayload();

	  // Initialise the library
	  JD8360RT.begin();

	  // Set the servo limits
	  // (Steering Servo Min, Steering Servo Max, Rear Link Servo Min, Rear Link Servo Max, Front Link Servo Min, Front Link Servo Max)
	  JD8360RT.SetServoLimits(0x70, 0x15, 0x10, 0x70, 0x00, 0xFF);

	  // Set the motor configuration
	  // (Motor configuration 1 = 1 motor or 2 = 2 motors or 3 = tracks, Do you use a PCF8574 0=no or 1=yes)
	  JD8360RT.MotorConfiguration(1, 0);

	  All_Controllers();

	  pinMode(A0,OUTPUT);
}

void loop()
{
	/* if (Controller_Locked == 0) {

	    // if there is data ready
	    if ( radio.available() )
	    {


	    }
	  }

	  if (Controller_Locked == 1) {
	   */ //Lock_Controller();
	    buf[0] = 0;

	    // if there is data ready
	    if ( radio.available() )
	    {
	      //Controller_ID = Controller_ID - 1;
	     // Lock_Controller();

	      long Vehicle_Voltage = JD8360RT.ReadVcc();

	        // Fetch the payload.
	        radio.read( buf, sizeof(buf));

	      // Add an ack packet for the next time around.  This is a simple
	      // packet counter
	      radio.writeAckPayload( 1, &Vehicle_Voltage, sizeof(Vehicle_Voltage));

	      // We recieved a string of ten bytes, four ID, one command and six data values but they may not be for this tractor
	      if (buf[0] == ID) { // Confirm that the correct vehicle ID has been recieved
	        Controller_ID = Controller_ID - 1;
	        //Lock_Controller();
	        JD8360RT.Lights(buf[9]);        // Update the lights
	        if(digitalRead(3)==1){digitalWrite(A0,HIGH);}else{digitalWrite(A0,LOW);}

	        JD8360RT.Trailer(buf);

	        Last_Data_Timer = 10000;
	        Active = 1;
	      }
	    }
	    else if (Last_Data_Timer != 0) {
	      Last_Data_Timer = Last_Data_Timer - 1;
	    }
	    else if (Last_Data_Timer == 0 && Active == 1) {
	      Active = 0;
	      JD8360RT.Lights(0x03);
	      JD8360RT.Drive(125, 125);
	    }

	    JD8360RT.Timer();


	    /* if (Active == 1) {
	      }
	      else if (Controller_Locked != 0) {
	       All_Controllers();
	       Controller_Locked = 0;
	      }*/
	  //}
	}

	void Lock_Controller() {
	/*  if (Controller_Locked == 0) {
	    radio.stopListening();

	    radio.openReadingPipe(1, Controller[Controller_ID]); // Set Address of Controller 1
	    radio.openReadingPipe(2, 0); // Set Address of Controller 2
	    radio.openReadingPipe(3, 0); // Set Address of Controller 1
	    radio.openReadingPipe(4, 0); // Set Address of Controller 2

	    radio.startListening();
	  }*/
	}

	void All_Controllers() {
	  radio.stopListening();

	  radio.openReadingPipe(1, Controller[0]); // Set Address of Controller 1
	  radio.openReadingPipe(2, Controller[1]); // Set Address of Controller 2
	  radio.openReadingPipe(3, Controller[2]); // Set Address of Controller 1
	  radio.openReadingPipe(4, Controller[3]); // Set Address of Controller 2
	  radio.openReadingPipe(5, Controller[4]); // Set Address of Controller 2

	  radio.startListening();
	}

